package io.openim.android.ouicore.widget;

import android.content.Context;
import android.content.res.TypedArray;
import android.text.Editable;
import android.text.TextWatcher;
import android.util.AttributeSet;
import android.view.MotionEvent;
import android.view.View;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.LinearLayout;

import androidx.annotation.Nullable;

import com.blankj.utilcode.util.StringUtils;

import io.openim.android.ouicore.R;
import io.openim.android.ouicore.databinding.LayoutSearchBinding;
import io.openim.android.ouicore.net.bage.Base;
import io.openim.android.ouicore.utils.L;

public class SearchView extends LinearLayout {
    boolean clickable = true, isClear;
    String hint;

    public SearchView(Context context) {
        super(context);
        initView();
    }

    public SearchView(Context context, @Nullable AttributeSet attrs) {
        super(context, attrs);
        TypedArray array = context.obtainStyledAttributes(attrs, R.styleable.customize);
        try {
            clickable = array.getBoolean(R.styleable.customize_android_clickable, true);
            hint = array.getString(R.styleable.customize_android_hint);
            isClear = array.getBoolean(R.styleable.customize_isClear, false);
        } finally {
            array.recycle();
        }
        initView();
    }

    public SearchView(Context context, @Nullable AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
        initView();
    }

    EditText editText;

    private void initView() {
        setBackground(getResources().getDrawable(R.drawable.sty_radius_2_f8f8f8));
        inflate(getContext(), R.layout.layout_search, this);
        editText = findViewById(R.id.editText);
        if (null != hint)
            editText.setHint(hint);
        ImageView imageView = findViewById(R.id.clearIv);
        imageView.setVisibility(isClear ? VISIBLE : GONE);
        imageView.setOnClickListener(this::onClick);
        editText.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence charSequence, int i, int i1, int i2) {

            }

            @Override
            public void onTextChanged(CharSequence charSequence, int i, int i1, int i2) {

            }

            @Override
            public void afterTextChanged(Editable editable) {
                imageView.setVisibility( StringUtils.isEmpty(editable.toString())?GONE : VISIBLE);

            }
        });


        if (clickable){
            editText.setFocusable(true);
            editText.setClickable(true);
            editText.setEnabled(true);
        }else {
            editText.setFocusable(false);
            editText.setClickable(false);
            editText.setEnabled(false);
        }
    }

    public EditText getEditText() {
        return editText;
    }

    @Override
    public boolean onInterceptTouchEvent(MotionEvent ev) {
        if (clickable)
            return super.onInterceptTouchEvent(ev);
        else
            return true;
    }

    private void onClick(View v) {
        editText.setText("");
    }
}
